using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using MovieDB.Common;
using MovieDB.Entities;
using MovieDB.Migration.Entities;

namespace MovieDB.Migration
{
    public class Migration
    {
        private readonly Dictionary<string, AudioQuality> _audioQualities;
        private readonly Dictionary<string, VideoQuality> _videoQualities;

        public Migration()
        {
            _audioQualities = new Dictionary<string, AudioQuality>();
            _videoQualities = new Dictionary<string, VideoQuality>();
        }

        public bool Migrate()
        {
            var accessDbConnectionString = ConfigurationManager.ConnectionStrings["MovieDBAccessEntities"].ConnectionString;
            var movieDbConnectionString = ConfigurationManager.ConnectionStrings["MovieDBEntities"].ConnectionString;

            using (var migrationsContext = new MigrationsContext(accessDbConnectionString))
            {
                var accessMovies = migrationsContext.MigrationsMovies.ToList();

                using (var movieContext = new MovieContext(movieDbConnectionString))
                {
                    foreach (Movies accessMovie in accessMovies)
                    {
                        var genre = GetGenre(accessMovie.Genre);
                        var videoQuality = GetVideoQuality(movieContext, accessMovie.Type);
                        var audioQuality = GetAudioQuality(movieContext, accessMovie.Audio_Stream);

                        var movieToCreate = CreateMovie(accessMovie.Titel, accessMovie.Nr, genre, videoQuality, audioQuality);
                        movieContext.Movies.Add(movieToCreate);
                    }
                    movieContext.SaveChanges();
                }
            }

            return false;
        }

        private Movie CreateMovie(string title, string location, MovieGenre genre, VideoQuality videoQuality, AudioQuality audioQuality)
        {
            return new Movie
                       {
                           Id = Guid.NewGuid(),
                           Title = title,
                           Genre = (int) genre,
                           Location = location,
                           AudioQualities = new List<AudioQuality> { audioQuality },
                           VideoQuality = videoQuality
                       };
        }

        private AudioQuality GetAudioQuality(MovieContext movieContext, string audioStream)
        {
            AudioQuality audioQuality;

            switch (audioStream)
            {
                case "DD2.0":
                case "DD21":
                    if (_audioQualities.ContainsKey(audioStream))
                    {
                        audioQuality = _audioQualities[audioStream];
                    }
                    else
                    {
                        audioQuality = movieContext.AudioQualities.Where(audio => audio.AudioType == (int) AudioType.DolbyDigital && audio.AudioChannels == (int) AudioChannels.TwoDotZero).Single();
                        _audioQualities.Add(audioStream, audioQuality);
                    }
                    break;
                case "DD51":
                    if (_audioQualities.ContainsKey(audioStream))
                    {
                        audioQuality = _audioQualities[audioStream];
                    }
                    else
                    {
                        audioQuality = movieContext.AudioQualities.Where(audio => audio.AudioType == (int) AudioType.DolbyDigital && audio.AudioChannels == (int) AudioChannels.FiveDotOne).Single();
                        _audioQualities.Add(audioStream, audioQuality);
                    }
                    break;

                case "DTS51":
                    if (_audioQualities.ContainsKey(audioStream))
                    {
                        audioQuality = _audioQualities[audioStream];
                    }
                    else
                    {
                        audioQuality = movieContext.AudioQualities.Where(audio => audio.AudioType == (int) AudioType.DTS && audio.AudioChannels == (int) AudioChannels.FiveDotOne).Single();
                        _audioQualities.Add(audioStream, audioQuality);
                    }
                    break;
                default:
                    throw new ArgumentOutOfRangeException("audioStream", "AudioStream is not supported");
            }

            return audioQuality;
        }


        private VideoQuality GetVideoQuality(MovieContext movieContext, string type)
        {
            VideoQuality videoQuality;

            switch (type)
            {
                case "LD-576p":
                    if (_videoQualities.ContainsKey(type))
                    {
                        videoQuality = _videoQualities[type];
                    }
                    else
                    {
                        videoQuality = movieContext.VideoQualities.Where(video => video.VideoType == (int) VideoType.LD && video.VideoResolution == (int) VideoResolution.FiveSevenSix).Single();
                        _videoQualities.Add(type, videoQuality);
                    }
                    break;
                case "HD-1080p":
                    if (_videoQualities.ContainsKey(type))
                    {
                        videoQuality = _videoQualities[type];
                    }
                    else
                    {
                        videoQuality = movieContext.VideoQualities.Where(video => video.VideoType == (int) VideoType.HD && video.VideoResolution == (int) VideoResolution.TenEigthyP).Single();
                        _videoQualities.Add(type, videoQuality);
                    }
                    break;

                case "HD-720p":
                    if (_videoQualities.ContainsKey(type))
                    {
                        videoQuality = _videoQualities[type];
                    }
                    else
                    {
                        videoQuality = movieContext.VideoQualities.Where(video => video.VideoType == (int) VideoType.HD && video.VideoResolution == (int) VideoResolution.SevenTwentyP).Single();
                        _videoQualities.Add(type, videoQuality);
                    }
                    break;
                default:
                    throw new ArgumentOutOfRangeException("type", "VideoStream is not supported");
            }

            return videoQuality;
        }

        private MovieGenre GetGenre(string genre)
        {
            switch (genre)
            {
                case "Action":
                    return MovieGenre.Action;
                case "Disney":
                    return MovieGenre.Disney;
                case "Drama":
                    return MovieGenre.Drama;
                case "Fantasy":
                    return MovieGenre.Fantasy;
                case "Horror":
                    return MovieGenre.Horror;
                case "Komdie":
                    return MovieGenre.Comedy;
                case "Love":
                    return MovieGenre.Love;
                case "Science Fiction":
                    return MovieGenre.ScienceFiction;
                case "Thriller":
                    return MovieGenre.Thriller;
                case "Weihnachten":
                    return MovieGenre.XMas;
                default:
                    throw new ArgumentOutOfRangeException("genre", "MovieGenre is not supported");
            }
        }
    }
}